within ;
package MOS5

  model HeartElasticity

    Modelica.Blocks.Interfaces.RealInput Tsyst "dlka systoly" annotation (Placement(
          transformation(extent={{-120,12},{-80,52}}), iconTransformation(extent={{-126,62},
              {-100,88}})));
    Modelica.Blocks.Interfaces.RealOutput Et "elasticita (torr/ml)" annotation (Placement(
          transformation(extent={{94,16},{114,36}}), iconTransformation(extent={{100,20},
              {120,40}})));
    Modelica.Blocks.Interfaces.RealInput T0 "cas poctku systoly"   annotation (Placement(
          transformation(extent={{-120,-30},{-80,10}}),iconTransformation(extent={{-126,18},
              {-100,44}})));

   parameter Real Esys= 1/0.4 "maximln (systolick) elesticita (torr/ml)";
   parameter Real Edias = 1/10 "minimln (diastolick) elasticita (torr/ml)";
  equation

    Et = if (time - T0 >= 0 and time - T0 <= Tsyst) then 
        Edias + (Esys - Edias)/2*(1 - cos(Modelica.Constants.pi*(time - T0)/Tsyst)) else 
       if (time - T0 < (3/2)*Tsyst) then 
    Edias + (Esys - Edias)/2*(1 + cos(2*Modelica.Constants.pi
      *(time - T0 - Tsyst)/Tsyst)) else 
      Edias;

    annotation (Icon(graphics={
          Rectangle(extent={{-100,100},{100,-100}}, lineColor={0,0,255}),
          Text(
            extent={{-100,122},{100,100}},
            lineColor={0,0,255},
            textString="Heart Elasticity"),
          Text(
            extent={{-114,36},{-42,24}},
            lineColor={0,0,255},
            textString="T0"),
          Text(
            extent={{-112,82},{-40,70}},
            lineColor={0,0,255},
            textString="Tsyst"),
          Text(
            extent={{56,36},{124,24}},
            lineColor={0,0,255},
            textString="Et"),
          Line(
            points={{-60,-20},{-20,86},{0,-92},{40,60},{60,-60},{102,28}},
            color={0,0,255},
            smooth=Smooth.Bezier),
          Text(
            extent={{-44,34},{130,66}},
            lineColor={0,0,255},
            textString="Elasticity")}), Diagram(graphics));
  end HeartElasticity;

  connector PressureFlow "Pressure[mmHg] and Flow[ml/min]"
    Real pressure( final quantity="Pressure");
    flow Real q( final quantity="Flow", final unit="ml/min") "flow";
    annotation (Documentation(revisions="<html>
<table>
<tr>
<td>Author:</td>
<td>Marek Matejak</td>
</tr>
<tr>
<td>Copyright:</td>
<td>In public domains</td>
</tr>
<tr>
<td>By:</td>
<td>Charles University, Prague, Czech Republic</td>
</tr>
<tr>
<td>Date of:</td>
<td>january 2009</td>
</tr>
</table>
</html>",
      info="<html>
<p><font style=\"font-size: 9pt; \">This connector connects hydraulic domains elements. The elements contains one equation for each his pressure-flow connector. The equation defines relation between variables in the connector. Variables are hydraulic pressure and volume flow of hydraulic medium. The pressure is the same in each connector that are connected together. The sum of flow in connectors connected together is zero (</font><b><font style=\"font-size: 9pt; \">Kirchhoff&apos;s circuit laws</font></b><font style=\"font-size: 9pt; \">).</font> </p>
</html>"));
  end PressureFlow;

  connector NegativePressureFlow "Pressure[mmHg] and negative Outflow[ml/min]"

    extends PressureFlow;

  annotation (
      defaultComponentName="q_out",
      Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{
              100,100}}), graphics={Rectangle(
            extent={{-20,10},{20,-10}},
            lineColor={0,0,0},
            lineThickness=1), Polygon(
            points={{0,100},{100,0},{0,-100},{-100,0},{0,100}},
            lineColor={0,0,0},
            smooth=Smooth.None,
            fillPattern=FillPattern.Solid,
            fillColor={200,200,200})}),
      Diagram(Polygon(points=[-21,-3; 5,23; 31,-3; 5,-29; -21,-3],   style(
            color=74,
            rgbcolor={0,0,0},
            fillColor=0,
            rgbfillColor={0,0,0})), Text(
          extent=[-105,-38; 115,-83],
          string="%name",
          style(color=0, rgbcolor={0,0,0}))));
  end NegativePressureFlow;

  connector PositivePressureFlow "Pressure[mmHg] and Inflow[ml/min]"
    extends PressureFlow;

  annotation (
      defaultComponentName="q_in",
      Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{
              100,100}}), graphics={Rectangle(
            extent={{-20,10},{20,-10}},
            lineColor={0,0,0},
            lineThickness=1), Polygon(
            points={{0,100},{100,0},{0,-100},{-100,0},{0,100}},
            lineColor={0,0,0},
            smooth=Smooth.None,
            fillPattern=FillPattern.Solid,
            fillColor={0,0,0})}),
      Diagram(Polygon(points=[-21,-3; 5,23; 31,-3; 5,-29; -21,-3],   style(
            color=74,
            rgbcolor={0,0,0},
            fillColor=0,
            rgbfillColor={0,0,0})), Text(
          extent=[-105,-38; 115,-83],
          string="%name",
          style(color=0, rgbcolor={0,0,0}))));
  end PositivePressureFlow;

  model TestHeartElasticity
    HeartElasticity heartElasticity 
      annotation (Placement(transformation(extent={{60,0},{80,20}})));
    Modelica.Blocks.Sources.Constant Tsyst(k=0.27) 
      annotation (Placement(transformation(extent={{-80,8},{-60,28}})));
    Modelica.Blocks.Sources.Constant T0(k=0.83) 
      annotation (Placement(transformation(extent={{-60,-20},{-40,0}})));
  equation
    connect(Tsyst.y, heartElasticity.Tsyst) annotation (Line(
        points={{-59,18},{-9.35,18},{-9.35,17.5},{58.7,17.5}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(T0.y, heartElasticity.T0) annotation (Line(
        points={{-39,-10},{-34,-10},{-34,13.1},{58.7,13.1}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics={Rectangle(extent={{-84,58},{-22,-24}},
              lineColor={0,0,255}), Text(
            extent={{-72,48},{-34,44}},
            lineColor={0,0,255},
            textString="HeartIntervals za domaci ukol")}));
  end TestHeartElasticity;

  model P
  parameter Real tlak = 0;
    NegativePressureFlow q_out 
      annotation (Placement(transformation(extent={{-10,-10},{10,10}}),
          iconTransformation(extent={{-10,-10},{10,10}})));
  equation
    q_out.pressure = tlak;
    annotation (Icon(graphics={Rectangle(extent={{-80,60},{60,-40}}, lineColor={0,
                0,255}), Text(
            extent={{-80,66},{62,100}},
            lineColor={0,0,255},
            textString="Pzdroj"),
          Text(
            extent={{-76,-98},{56,-44}},
            lineColor={0,0,255},
            textString="%tlak")}));
  end P;

  model Odpor

    PositivePressureFlow q_in annotation (Placement(transformation(extent={{-90,-10},
              {-70,10}}), iconTransformation(extent={{-90,-10},{-70,10}})));
    NegativePressureFlow q_out annotation (Placement(transformation(extent={{70,-10},
              {90,10}}), iconTransformation(extent={{70,-10},{90,10}})));
  parameter Real R = 1;
  Real dp;
  equation
    q_in.q + q_out.q = 0;
    dp = q_in.pressure - q_out.pressure;
    dp = R*q_in.q;
    annotation (Icon(graphics={Rectangle(extent={{-60,20},{60,-20}}, lineColor={0,
                0,255})}));
  end Odpor;

  model chlopen
  Real s;
  Boolean off;
  Real dp;
    PositivePressureFlow q_in 
      annotation (Placement(transformation(extent={{-90,-10},{-70,10}})));
    NegativePressureFlow q_out 
      annotation (Placement(transformation(extent={{70,-10},{90,10}})));
  equation
  off = s <0;
  q_in.q = if off then 0 else s;
  dp = q_in.pressure - q_out.pressure;
  dp = if off then s else 0;
  q_in.q + q_out.q = 0;
    annotation (Icon(graphics={Polygon(
            points={{-60,60},{40,0},{-60,-60},{-60,-20},{-60,60}},
            lineColor={127,0,0},
            smooth=Smooth.None,
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid), Rectangle(
            extent={{40,60},{60,-60}},
            lineColor={127,0,0},
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid)}));
  end chlopen;

  model komora

    PositivePressureFlow q_in 
      annotation (Placement(transformation(extent={{-10,-90},{10,-70}})));
    Modelica.Blocks.Interfaces.RealInput elastance annotation (Placement(transformation(
            extent={{-60,-32},{-20,8}}), iconTransformation(extent={{-60,-32},{-20,
              8}})));
  parameter Real pExt = -4;
  Real volume;
  Real pTransmural;
  equation
    der(volume) = q_in.q;
    pTransmural = volume*elastance;
    q_in.pressure = pTransmural + pExt;

    annotation (Diagram(graphics={Ellipse(
            extent={{-60,60},{60,-80}},
            lineColor={127,0,0},
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid)}), Icon(graphics={Ellipse(
            extent={{-60,60},{60,-80}},
            lineColor={127,0,0},
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid)}));
  end komora;

  model testHeartBeat
    P pZdroj annotation (Placement(transformation(extent={{-96,18},{-76,38}})));
    Odpor odporVstup 
      annotation (Placement(transformation(extent={{-66,18},{-46,38}})));
    chlopen chlopen1 
      annotation (Placement(transformation(extent={{-30,18},{-10,38}})));
    komora komora1 
      annotation (Placement(transformation(extent={{20,44},{40,64}})));
    Odpor odporVystup 
      annotation (Placement(transformation(extent={{34,18},{54,38}})));
    P pOdtok(tlak=12) 
      annotation (Placement(transformation(extent={{68,18},{88,38}})));
    chlopen chlopen2 
      annotation (Placement(transformation(extent={{8,18},{28,38}})));
    Modelica.Blocks.Sources.Ramp ramp(
      height=120,
      duration=20,
      offset=60,
      startTime=10) 
      annotation (Placement(transformation(extent={{-92,68},{-72,88}})));
    HeartElasticity heartElasticity 
      annotation (Placement(transformation(extent={{-18,66},{2,86}})));
  equation
    connect(pZdroj.q_out, odporVstup.q_in) annotation (Line(
        points={{-86,28},{-64,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(odporVstup.q_out, chlopen1.q_in) annotation (Line(
        points={{-48,28},{-28,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(odporVystup.q_out, pOdtok.q_out) annotation (Line(
        points={{52,28},{78,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(chlopen2.q_out, odporVystup.q_in) annotation (Line(
        points={{26,28},{36,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(chlopen1.q_out, chlopen2.q_in) annotation (Line(
        points={{-12,28},{10,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(komora1.q_in, chlopen2.q_in) annotation (Line(
        points={{30,46},{6,46},{6,28},{10,28}},
        color={0,0,0},
        thickness=1,
        smooth=Smooth.None));
    connect(heartElasticity.Et, komora1.elastance) annotation (Line(
        points={{3,79},{14,79},{14,52.8},{26,52.8}},
        color={0,0,127},
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end testHeartBeat;

  model testElasticity
    HeartElasticity heartElasticity 
      annotation (Placement(transformation(extent={{26,-38},{84,18}})));
    heartIntervals heartIntervals1 
      annotation (Placement(transformation(extent={{-236,8},{-72,96}})));
    Modelica.Blocks.Sources.Constant const(k=60) 
      annotation (Placement(transformation(extent={{-228,120},{-208,140}})));
  equation
    connect(heartIntervals1.T0, heartElasticity.T0) annotation (Line(
        points={{-80.2,32.64},{-30,32.64},{-30,-1.32},{22.23,-1.32}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(heartIntervals1.TSyst, heartElasticity.Tsyst) annotation (Line(
        points={{-80.2,59.04},{-17.1,59.04},{-17.1,11},{22.23,11}},
        color={0,0,127},
        smooth=Smooth.None));
    connect(const.y, heartIntervals1.hr) annotation (Line(
        points={{-207,130},{-196,130},{-196,110},{-264,110},{-264,69.6},{-219.6,
            69.6}},
        color={0,0,127},
        smooth=Smooth.None));

    annotation (Diagram(graphics));
  end testElasticity;


  annotation (uses(Modelica(version="3.1")));
end MOS5;
